/*-------------- Telecommunications & Signal Processing Lab ---------------
                             McGill University

Routine:
  struct AF_info *AFgenHinfo (double Sfreq)

Purpose:
  Form an AFsp information structure

Description:
  This returns an AFsp information structure.  The information records in
  that structure are in a form suitable for writing directly to an audio file
  header.  The header string is formed from a combination of the standard
  header information and the user supplied information string.  There are three
  cases.
    - User header information not specified.  The standard header information
      records are generated.
    - User header information string starts with a newline.  The standard
      header information is generated and then the user header information
      string (containing information records) is appended.
    - User header information string does not start with a newline.  Only the
      user header information string is used.

  Standard Header Information:
    date: 1994-01-25 19:19:39 UTC  date
    sample_rate: 8012.5            sampling frequency (only if non-integer)
    user: kabal@aldebaran          user
    program: CopyAudio             program name (if set via routine UTsetProg)

Parameters:
  <-  struct AF_info *AFgenHinfo
      AFsp information structure.  The information records in the structure
      are in a form that can be directly written to the header, i.e. newline
      characters have been changed to null characters.  This string is
      allocated by this routine.
   -> double Sfreq
      Sampling frequency.  If this value is non-integer, a sample_rate record
      is put in the standard header information string.

Author / revision:
  P. Kabal  Copyright (C) 2000

  $Revision: 1.24 $  $Date: 2000/11/21 14:53:28 $

-------------------------------------------------------------------------*/

static char rcsid[] = "$Id: AFgenHinfo.c 1.24 2000/11/21 AFsp-v5r2 $";

#include <assert.h>
#include <math.h>	/* floor */
#include <string.h>

#include <libtsp.h>
#include <libtsp/AFheader.h>
#include <libtsp/AFpar.h>

#define MAXV(a, b)	(((a) > (b)) ? (a) : (b))

#define MAX_SINFO	320
#define MIN_HINFO	100

static int
AF_stdInfo (char *Sinfo, double Sfreq);
static int
AF_nlNull (char *Info);


struct AF_info *
AFgenHinfo (double Sfreq)

{
  char Sinfo[MAX_SINFO];
  char *Uinfo;
  int Nc, ns, nu;
  static struct AF_info Hinfo = {NULL, 0, 0};

  Uinfo = AFgetHinfo ();
  if (Uinfo == NULL || Uinfo[0] == '\n')
    ns = AF_stdInfo (Sinfo, Sfreq);
  else {
    ns = 0;
    Sinfo[0] = '\0';
  }

/* Size of the user specified header information */
  if (Uinfo != NULL)
    nu = strlen (Uinfo);
  else
    nu = 0;

/* Allocate storage (not less than MIN_HINFO) */
  Nc = MAXV (MIN_HINFO, nu + ns + 1);	/* Leave room for a null */
  if (Hinfo.Nmax != Nc) {
    UTfree ((void *) Hinfo.Info);
    Hinfo.Info = (char *) UTmalloc (Nc);
    Hinfo.Nmax = Nc;
  }

/* Form the output string.
   - Uinfo == NULL
       Info <= Sinfo
   - Uinfo is empty
       Info <= empty
   - Uinfo starts with '\n'
       Info <= Sinfo + Uinfo
       In this case, the leading '\n' in Uinfo overlays the trailing null
       in Sinfo.
   - Uinfo does not start with '\n'
       Info <= Uinfo
*/
  strcpy (Hinfo.Info, Sinfo);
  if (Uinfo != NULL)
    strcpy (&Hinfo.Info[ns], Uinfo);

/* Change newlines to nulls */
  Hinfo.N = AF_nlNull (Hinfo.Info);

/* Return a pointer to the header information */
  return &Hinfo;
}

/* Generate the standard header information string */
/* "date: xxx"
   + "\n" + "sample_rate: xxx" (only if non-integer)
   + "\n" + "user: xxx"
   + "\n" "program: xxx"

   Lines are separated by newline characters.  The overall string is null
   terminated.  The returned function value is the number of characters not
   including the terminating null.
*/


static int
AF_stdInfo (char *Sinfo, double Sfreq)

{
  int N;
  char *p;

  sprintf (Sinfo, "date: %.40s", UTdate(3));
  N = strlen (Sinfo);	/* Some versions of sprintf return a pointer */
  if (Sfreq > 0.0 && Sfreq != floor (Sfreq)) {
    sprintf (&Sinfo[N], "\nsample_rate: %.7g", Sfreq);
    N += strlen (&Sinfo[N]);
  }
  p = UTuserName ();
  if (*p != '\0') {
    sprintf (&Sinfo[N], "\nuser: %.40s", p);
    N += strlen (&Sinfo[N]);
  }
  p = UTgetProg ();
  if (*p != '\0') {
    sprintf (&Sinfo[N], "\nprogram: %.40s", p);
    N += strlen (&Sinfo[N]);
  }
  assert (N < MAX_SINFO);

  return N;
}

/* Change newlines to nulls (unless escaped) */
/* - Escaped characters.  In Unix it is relatively easy to get a newline
     into a command line string.  In Windows, one can use the escape
     sequences instead.
     - \n    => <nul>  newline
     - <nl>  => <nul>  newline
     - \r    => <nl>   escaped newline
     - \<nl> => <nl>   escaped newline
   - If the length of the input string is not zero and the last character in
     the input string was not a newline (changed to a null), a null character
     is added to the string.
   - The count of the number of characters includes the null character at the
     end of the string.
   - If the number of input characters is zero, the return value is zero.
*/


static int
AF_nlNull (char *Info)

{
  int i, n;

  for (i = 0, n = 0; Info[i] != '\0'; ++i, ++n) {
    if (Info[i] == '\\' && Info[i+1] == 'n') {
      Info[n] = '\0';
      ++i;
    }
    else if (Info[i] == '\n')
      Info[n] = '\0';
    else if (Info[i] == '\\' && Info[i+1] == 'r') {
      Info[n] = '\n';
      ++i;
    }
    else if (Info[i] == '\r')
      Info[n] = '\n';
    else if (Info[i] == '\\' && Info[i+1] == '\n') {
      Info[n] = '\n';
      ++i;
    }
    else if (n != i)
      Info[n] = Info[i];
  }

  /* Add a terminating null (included in the character count) if one is not
     present */
  if (n > 0 && Info[n-1] != '\0') {
    Info[n] = '\0';
    ++n;
  }

  return n;
}
